from setuptools import setup

dependencies = [
    "blspy==1.0.2",  # Signature library
    "chiavdf==1.0.2",  # timelord and vdf verification
    "chiabip158==1.0",  # bip158-style wallet filters
    "chiapos==1.0.3",  # proof of space
    "clvm==0.9.7",
    "clvm_rs==0.1.8",
    "clvm_tools==0.4.3",
    "aiohttp==3.7.4",  # HTTP server for full node rpc
    "aiosqlite==0.17.0",  # asyncio wrapper for sqlite, to store blocks
    "bitstring==3.1.7",  # Binary data management library
    "colorlog==5.0.1",  # Adds color to logs
    "concurrent-log-handler==0.9.19",  # Concurrently log and rotate logs
    "cryptography==3.4.7",  # Python cryptography library for TLS - keyring conflict
    "keyring==23.0.1",  # Store keys in MacOS Keychain, Windows Credential Locker
    "keyrings.cryptfile==1.3.4",  # Secure storage for keys on Linux (Will be replaced)
    #  "keyrings.cryptfile==1.3.8",  # Secure storage for keys on Linux (Will be replaced)
    #  See https://github.com/frispete/keyrings.cryptfile/issues/15
    "PyYAML==5.4.1",  # Used for config file format
    "setproctitle==1.2.2",  # Gives the flora processes readable names
    "sortedcontainers==2.3.0",  # For maintaining sorted mempools
    "websockets==8.1.0",  # For use in wallet RPC and electron UI
    "click==7.1.2",  # For the CLI
    "dnspython==2.1.0",  # Query DNS seeds
]

upnp_dependencies = [
    "miniupnpc==2.2.2",  # Allows users to open ports on their router
]

dev_dependencies = [
    "pytest",
    "pytest-asyncio",
    "flake8",
    "mypy",
    "black",
    "aiohttp_cors",  # For blackd
    "ipython",  # For asyncio debugging
]

kwargs = dict(
    name="flora-blockchain",
    version="0.2.5",
    author="Mariano Sorgente",
    author_email="mariano@flora.net",
    description="Flora blockchain full node, farmer, timelord, and wallet.",
    url="https://floracoin.farm",
    license="Apache License",
    python_requires=">=3.7, <4",
    keywords="flora blockchain node",
    install_requires=dependencies,
    setup_requires=["setuptools_scm"],
    extras_require=dict(
        uvloop=["uvloop"],
        dev=dev_dependencies,
        upnp=upnp_dependencies,
    ),
    packages=[
        "build_scripts",
        "flora",
        "flora.cmds",
        "flora.clvm",
        "flora.consensus",
        "flora.daemon",
        "flora.full_node",
        "flora.timelord",
        "flora.farmer",
        "flora.harvester",
        "flora.introducer",
        "flora.plotting",
        "flora.pools",
        "flora.protocols",
        "flora.rpc",
        "flora.server",
        "flora.simulator",
        "flora.types.blockchain_format",
        "flora.types",
        "flora.util",
        "flora.wallet",
        "flora.wallet.puzzles",
        "flora.wallet.rl_wallet",
        "flora.wallet.cc_wallet",
        "flora.wallet.did_wallet",
        "flora.wallet.settings",
        "flora.wallet.trading",
        "flora.wallet.util",
        "flora.ssl",
        "mozilla-ca",
    ],
    entry_points={
        "console_scripts": [
            "flora = flora.cmds.flora:main",
            "flora_wallet = flora.server.start_wallet:main",
            "flora_full_node = flora.server.start_full_node:main",
            "flora_harvester = flora.server.start_harvester:main",
            "flora_farmer = flora.server.start_farmer:main",
            "flora_introducer = flora.server.start_introducer:main",
            "flora_timelord = flora.server.start_timelord:main",
            "flora_timelord_launcher = flora.timelord.timelord_launcher:main",
            "flora_full_node_simulator = flora.simulator.start_simulator:main",
        ]
    },
    package_data={
        "flora": ["pyinstaller.spec"],
        "flora.wallet.puzzles": ["*.clvm", "*.clvm.hex"],
        "flora.util": ["initial-*.yaml", "english.txt"],
        "flora.ssl": ["flora_ca.crt", "flora_ca.key", "dst_root_ca.pem"],
        "mozilla-ca": ["cacert.pem"],
    },
    #use_scm_version={"fallback_version": "0.2.4"},
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    zip_safe=False,
)


if __name__ == "__main__":
    setup(**kwargs)
